/*----------------------------------------------------------------------------* 
**  "comp2_2.cpp" using Trapezoidal Integration
**
**  To build this in Visual C++ 6.0 perform the following steps:
**
**  1. Select New from File menu.
**
**  2. In the New dialog box select "Win32 Console Application" and enter 
**     a project name.  Then press the "Ok" button.
**
**  3. In the Win32 Application dialog box select "A simple application" 
**     then press the "Finish" button.  
**
**  4. Press "Ok" in the New Project Information dialog box.
**
**  5. Open the *.cpp file for entry of the program.  (We do not edit StdAfx.h
**     or StdAfx.cpp; VC++ requires these files.)
*/

#include "stdafx.h"
#include <stdio.h>
#include <math.h>

double integrate( double, double, int, double (f)(double) );
double Y( double );

/*----------------------------------------------------------------------------* 
**  "main" test of IntegTrpz.
**
**  returns: 0
*/
int main()
{
    double a;                                   // approximate integral
    int    n, i;

    n = 0;                                      // init number of segments

    for( i = 1; i <= 10; i++ )
    {
	n += 10;
	a = integrate( 0.0, 3.0, n, Y );
	printf( "%4d  %12.9f\n", n, a );
    }

    return 0;
}

/*----------------------------------------------------------------------------* 
**  "Y" test function.
**
**  returns: value at time t
*/
double Y( double t )
{
    return fabs( sin( t ) ) / ( 3.0 * t + 1.0 );
}

/*----------------------------------------------------------------------------* 
**  "integrate" finds the definite integral of a specified function
**  using trapezoidal integration.
**
**  returns: nothing
*/
double integrate( double t_beg,                 // lower limit of integral
	          double delta,                 // period for integration
		  int    n,                     // number of segments > 0 
		  double (f)(double t) )        // integrand function
{
    double area;
    double t = t_beg;                           // initialize time
    double dt = delta / n;                      // initialize step size
    int    i;                                   // loop counter

    area = f( t_beg ) / 2.0;                    // initial value of integral 

    if( n > 1 )                                 // compute the n - 1 terms
    {                                           //   in the summation
	for( i = 1; i < n; i++ )
	{
	    t += dt;                            // increment time
	    area += f( t );                     // add next term to sum
	}
    }

    area += f( t_beg + delta ) / 2.0;           // add last term
    area *= dt;                                 // scale by delta-time

    return area;
}

