/*----------------------------------------------------------------------------* 
**  "comp5_2.cpp"  using the Runge-Kutta numerical differentiation method
**  for solving differential equations.
**
**  To build this in Visual C++ 6.0 perform the following steps:
**
**  1. Select New from File menu.
**
**  2. In the New dialog box select "Win32 Application" and enter 
**     a project name.  Then press the "Ok" button.
**
**  3. In the Win32 Application dialog box select "A simple Win32 Application" 
**     then press the "Finish" button.  
**
**  4. Press "Ok" in the New Project Information dialog box.
**
**  5. Open the *.cpp file for entry of the program.  (We do not edit StdAfx.h
**     or StdAfx.cpp; VC++ requires these files.)
*/

#include "stdafx.h"
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <windowsx.h>
#include "..\plotter\plotter.h"
#include "..\vector\vector.h"

plotter np( "Plotter", "Numerical Plotter" ); 

/*----------------------------------------------------------------------------*
**  "WinMain" is the program entry point
**
**  returns: message parameter
*/
int APIENTRY WinMain( HINSTANCE inst,           // window instance
                      HINSTANCE prev,           // previous window
                      LPSTR     cmdl,           // command line string
                      int       show )          // initial display
{
    np.initialize( inst );
    
    if( !np.window_class() )
	return 0;
  
    np.create( show );
 
    return np.message_loop();
}
    
/*----------------------------------------------------------------------------*
**  "WindowFunc" processes messages to this application
**
**  returns: 0 if message processed, else default value
*/
LRESULT CALLBACK WindowFunc( HWND   hwnd,       // handle to window of message
			     UINT   msg,        // message code
			     WPARAM wp,         // short message amplifier
			     LPARAM lp )        // long message amplifier
{
    switch( msg )
    {
    case WM_CREATE:
	np.on_create( hwnd );
	break;

    case WM_DESTROY:
	np.on_destroy();
	break;

    case WM_SIZE:
	np.on_size( hwnd );
	break;

    case WM_HSCROLL:
	np.on_hscroll( hwnd, wp );
	break;

    case WM_VSCROLL:
	np.on_vscroll( hwnd, wp );
	break;

    case WM_PAINT:
	np.on_paint( hwnd );
	break;

    case WM_COMMAND:
	np.on_command( hwnd, wp );
	break;

    default:
	return DefWindowProc( hwnd, msg, wp, lp );
    }

    return 0;
}

/*----------------------------------------------------------------------------*\
**              Put your mathematical functions to plot here.                 **
\*----------------------------------------------------------------------------*/

int calculate( POINT *, int );
double rk( double, double, double, int, double (g)(double, double) );
double G( double, double );

/*----------------------------------------------------------------------------*
**  "run" performs calculations and plots results.  
**
**  This is a shell in which the function calls for a specific calculation
**  are entered.
**
**  returns: nothing
*/
void run()
{
    POINT curve[100];
    int i;
            
    // first, define a plot region 
    // this must be done before calculating points to be plotted
    np.plot_region( -0.2, 5.0, 11, -4 );

    i = calculate( curve, 100 );

    // plot the results
    np.draw_axes();   
    np.red_curve( curve, i );
}
    
/*----------------------------------------------------------------------------*
**  "calculate" is a shell for the calculation algorithms.
**
**  returns: number of items in POINT array
*/
int calculate( POINT *a0, int points )
{
    int i;
    double y     = 10.0; 
    double t     = 0.05;
    double delta = 0.05;

//    a1[0].x = int( ( X_orig_pixel + t * dX ) ); 
//    a1[0].y = int( ( Y_orig_pixel - y * dY ) );

    a0[0] = np.plot_point( t, y );

    for( i = 1; i < points; i++ )
    {
	// next iteration
	y = rk( t, y, delta, 2, G );
	t = t + delta;
	
	// append results to curves
	a0[i] = np.plot_point( t, y );
//	a1[i].x = int( ( X_orig_pixel + t * dX ) ); 
//	a1[i].y = int( ( Y_orig_pixel - y * dY ) );
    }
    return i;
}

/*----------------------------------------------------------------------------* 
**  "rk" performs numerical differentiation using the 4-th order 
**  Runge Kutta algorithm to solve equations of the form: 
**
**                      y' = g( t, y )
**
**  returns: the value of y at time t_end
*/
double rk( double t_beg,                        // interval start
	   double y_beg,                        // derivative at start
	   double delta,                        // interval length
	   int    n,                            // number of segments > 0 
	   double (g)(double, double) )         // integrand function
{
    double g1, g2, g3, g4;
    double t = t_beg;                           // initialize time
    double y = y_beg;
    double dt = delta / n;                      // initialize segment period
    int    i;                                   // loop counter

    for( i = 0; i < n; i++ )
    {
	g1 = g( t,            y );
	g2 = g( t + dt / 2.0, y + dt * g1 / 2.0 );
	g3 = g( t + dt / 2.0, y + dt * g2 / 2.0 );
	g4 = g( t + dt,       y + dt * g3 );

	y += ( g1 + 2.0 * g2 + 2.0 * g3 + g4 ) * dt / 6.0;
	t += dt;
    }

    return y;
}

double G( double t, double y )
{
    return ( 1.0 - y ) / t;
}


